package com.bwie.gateway.filters;

import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.SysUser;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import com.bwie.gateway.config.IgnoreWhiteConfig;
import com.bwie.gateway.utils.GatewayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @ClassName:
 * @Description: 鉴权过滤器  拦截请求验证 token
 * @Author: zhuwenqiang
 * @Date: 2023/10/19
 */
@Component
public class AuthFilter implements GlobalFilter, Ordered {

    @Autowired
    private IgnoreWhiteConfig ignoreWhitesConfig;

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    /**
     * 过滤方法 只要在这个方法中实现业务  验证 token
     * <p>
     * 会将不拦截的请求 配置到 配置文件中
     *
     * @param exchange 请求的上下文 获取请求和响应对象
     * @param chain    网关过滤器链 放行
     * @return
     */
    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        // 获取系统白名单请求
        List<String> whites = ignoreWhitesConfig.getWhites();
        // 判断 当前 请求是否是 白名单的请求
        // 获取当前请求的 URI
        ServerHttpRequest request = exchange.getRequest();
        String path = request.getURI().getPath();
        if (StringUtils.matches(path, whites)) {
            // 白名单请求 放行
            return chain.filter(exchange);
        }
        // 验证 token 非空
        String token = request.getHeaders().getFirst(TokenConstants.TOKEN);
        if (StringUtils.isBlank(token)) {
            // 提示错误信息
            return GatewayUtils.errorResponse(exchange, "token不能为空！");
        }
        // 合法性验证
        String userKey = null;
        try {
            userKey = JwtUtils.getUserKey(token);
        } catch (Exception e) {
            return GatewayUtils.errorResponse(exchange, "token不合法！");
        }
        // token 有效性验证
        // 获取 userKey
        if (!redisTemplate.hasKey(TokenConstants.LOGIN_TOKEN_KEY + userKey)) {
            // 提示错误信息
            return GatewayUtils.errorResponse(exchange, "token过期！");
        }
        // 续期  登录在10分钟以内自动续期
        // 获取用户的登录时间
        String userJSON = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        SysUser sysUser = JSONObject.parseObject(userJSON, SysUser.class);
        Date loginDate = sysUser.getLoginDate();
        long min = DateUtil.between(loginDate, new Date(), DateUnit.MINUTE);
        if (min <= 10) {
            redisTemplate.expire(TokenConstants.LOGIN_TOKEN_KEY + userKey, 30, TimeUnit.MINUTES);
        }
        // 放行
        return chain.filter(exchange);
    }

    /**
     * 过滤器的执行顺序 【过滤器可以有多个】  返回的值越小 优先级越高
     *
     * @return
     */
    @Override
    public int getOrder() {
        return 0;
    }
}
